/*
 * $Id: MBeanHelper.java,v 1.4 2009/12/19 22:34:09 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * (c)reated 19.02.2009 by oliver (ob@oasd.de)
 */
package patterntesting.runtime.jmx;

import java.lang.management.ManagementFactory;

import javax.management.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

/**
 * This class simplifies the use of JMX and MBeans a little bit.
 *
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 19.02.2009
 * @version $Revision: 1.4 $
 */
public class MBeanHelper {

    private static final Log log = LogFactoryImpl.getLog(MBeanHelper.class);
    private static MBeanServer server = ManagementFactory.getPlatformMBeanServer();

    /**
     * Gets the m bean name.
     *
     * @param mbean the mbean
     *
     * @return the name of the MBean
     */
    public static String getMBeanName(Object mbean) {
        return getMBeanName(mbean.getClass());
    }

    /**
     * Converts the class name into "p.a.c.k.a.g.e:type=Classname".
     *
     * @param cl e.g. my.hello.World
     *
     * @return e.g. "my.hello:type=World"
     */
    public static String getMBeanName(Class<?> cl) {
        return cl.getPackage().getName() + ":type=" + cl.getSimpleName();
    }

    /**
     * Register m bean.
     *
     * @param mbean the mbean
     *
     * @throws JMException
     */
    public static void registerMBean(Object mbean) throws JMException {
        String mbeanName = getMBeanName(mbean);
        registerMBean(mbeanName, mbean);
    }

    /**
     * Register m bean.
     *
     * @param mbeanName the mbean name
     * @param mbean the mbean
     */
    public static synchronized void registerMBean(String mbeanName, Object mbean) {
        try {
            ObjectName name = new ObjectName(mbeanName);
            registerMBean(name, mbean);
        } catch (MalformedObjectNameException e) {
            log.info("can't register <" + mbean + "> as MBean", e);
        }
    }

    /**
     * Register m bean.
     *
     * @param name the name
     * @param mbean the mbean
     */
    public static synchronized void registerMBean(ObjectName name, Object mbean) {
        try {
            log.trace("registering " + name + "...");
            server.registerMBean(mbean, name);
            log.debug(name + " successful registered as MBean");
        } catch (Exception e) {
            log.info("can't register <" + mbean + "> as MBean", e);
        }
    }

    /**
     * Unregister m bean.
     *
     * @param mbeanName the mbean name
     */
    public static synchronized void unregisterMBean(String mbeanName) {
        try {
            ObjectName name = new ObjectName(mbeanName);
            server.unregisterMBean(name);
            log.debug("MBean " + name + " successful unregistered");
        } catch (Exception e) {
            log.info("can't unregister " + mbeanName, e);
        }
    }

    /**
     * Checks if is registered.
     *
     * @param mbeanName the mbean name
     *
     * @return true, if is registered
     */
    public static boolean isRegistered(String mbeanName) {
        try {
            ObjectName name = new ObjectName(mbeanName);
            ObjectInstance mbean = server.getObjectInstance(name);
            return (mbean != null);
        } catch (Exception e) {
            log.trace(mbeanName + " not found (" + e + ")");
            return false;
        }
    }

    /**
     * Checks if is registered.
     *
     * @param obj the obj
     *
     * @return true, if is registered
     */
    public static boolean isRegistered(Object obj) {
        String mbeanName = getMBeanName(obj.getClass());
        return isRegistered(mbeanName);
    }

}

/**
 * $Log: MBeanHelper.java,v $
 * Revision 1.4  2009/12/19 22:34:09  oboehm
 * trailing spaces removed
 *
 * Revision 1.3  2009/09/25 14:49:43  oboehm
 * javadocs completed with the help of JAutodoc
 *
 * Revision 1.2  2009/03/22 13:56:58  oboehm
 * begin with next snapshot
 * isRegistered(Object) added
 *
 * Revision 1.1  2009/02/20 21:34:03  oboehm
 * MBeanRegistry with default implementation added
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-rt/src/main/java/patterntesting/runtime/jmx/MBeanHelper.java,v $
 */
